@@ -1,10 +1,19 @@
 //! Utilities for working with upstream versions.
 
+use std::borrow::Cow;
+
 static DFSG_REGEX: &lazy_regex::Lazy<lazy_regex::Regex> =
     lazy_regex::regex!(r"^(.*)([\+~])(dfsg|ds)([0-9]*)$");
 const DFSG_DEFAULT_STYLE: &str = "+ds";
 
 /// Strip the DFSG suffix from a version.
+///
+/// # Example
+/// ```
+/// use debversion::upstream::strip_dfsg_suffix;
+/// let version = "1.2.3+dfsg1";
+/// assert_eq!(strip_dfsg_suffix(version), Some("1.2.3"));
+/// ```
 pub fn strip_dfsg_suffix(version: &str) -> Option<&str> {
     if let Some(m) = DFSG_REGEX.captures(version) {
         Some(m.get(1).unwrap().as_str())
@@ -17,17 +26,30 @@ pub fn strip_dfsg_suffix(version: &str) -> Option<&str> {
 ///
 /// Allow old_upstream_version to be passed in so optionally the format can be
 /// kept consistent.
+///
+/// # Example
+/// ```
+/// use debversion::upstream::add_dfsg_suffix;
+/// assert_eq!(add_dfsg_suffix("1.2.3", None), "1.2.3+ds");
+/// assert_eq!(add_dfsg_suffix("1.2.3", Some("1.2.2+dfsg1")), "1.2.3+dfsg1");
+/// assert_eq!(add_dfsg_suffix("1.2.3", Some("1.2.2+ds1")), "1.2.3+ds1");
+/// assert_eq!(add_dfsg_suffix("1.2.3", Some("1.2.3")), "1.2.3+ds");
+/// ```
 pub fn add_dfsg_suffix(upstream_version: &str, old_upstream_version: Option<&str>) -> String {
     let style = if let Some(m) = old_upstream_version.and_then(|d| DFSG_REGEX.captures(d)) {
-        let mut style = m.get(2).unwrap().as_str().to_string() + m.get(3).unwrap().as_str();
-        if !m.get(4).unwrap().as_str().is_empty() {
-            style.push('1');
-        }
-        style
+        let part2 = m.get(2).unwrap().as_str();
+        let part3 = m.get(3).unwrap().as_str();
+        let part4 = m.get(4).unwrap().as_str();
+
+        Cow::Owned(if part4.is_empty() {
+            format!("{}{}", part2, part3)
+        } else {
+            format!("{}{}1", part2, part3)
+        })
     } else {
-        DFSG_DEFAULT_STYLE.to_string()
+        Cow::Borrowed(DFSG_DEFAULT_STYLE)
     };
-    upstream_version.to_string() + style.as_str()
+    format!("{}{}", upstream_version, style)
 }
 
 #[derive(Debug, Clone, PartialEq)]
@@ -64,36 +86,32 @@ impl VcsSnapshot {
                 date,
                 sha,
                 snapshot,
-            } => {
-                let decoded_gitid = sha.as_ref().map(|sha| &sha[..std::cmp::min(sha.len(), 7)]);
-                let gitdate = date.map(|d| d.format("%Y%m%d").to_string());
-                if let (Some(decoded_gitid), Some(snapshot), Some(gitdate)) =
-                    (decoded_gitid, snapshot, gitdate.as_ref())
-                {
-                    format!("git{}.{}.{}", gitdate, snapshot, decoded_gitid)
-                } else if let (Some(decoded_gitid), Some(gitdate)) =
-                    (decoded_gitid, gitdate.as_ref())
-                {
-                    format!("git{}.{}", gitdate, decoded_gitid)
-                } else if let Some(decoded_gitid) = decoded_gitid {
-                    format!("git{}", decoded_gitid)
-                } else if let Some(gitdate) = gitdate {
-                    format!("git{}", gitdate)
-                } else {
-                    "git".to_string()
+            } => match (sha.as_deref(), snapshot, date.as_ref()) {
+                (Some(sha), Some(snapshot), Some(date)) => {
+                    let gitid = &sha[..sha.len().min(7)];
+                    format!("git{}.{}.{}", date.format("%Y%m%d"), snapshot, gitid)
                 }
-            }
-            VcsSnapshot::Bzr { revno } => {
-                format!("bzr{}", revno)
-            }
-            VcsSnapshot::Svn { revno } => {
-                format!("svn{}", revno)
-            }
+                (Some(sha), None, Some(date)) => {
+                    let gitid = &sha[..sha.len().min(7)];
+                    format!("git{}.{}", date.format("%Y%m%d"), gitid)
+                }
+                (Some(sha), _, None) => {
+                    let gitid = &sha[..sha.len().min(7)];
+                    format!("git{}", gitid)
+                }
+                (None, _, Some(date)) => {
+                    format!("git{}", date.format("%Y%m%d"))
+                }
+                (None, _, None) => "git".to_string(),
+            },
+            VcsSnapshot::Bzr { revno } => format!("bzr{}", revno),
+            VcsSnapshot::Svn { revno } => format!("svn{}", revno),
         }
     }
 }
 
 /// Direction to add the snapshot.
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Direction {
     /// Snapshot predates the version.
     Before,
@@ -112,16 +130,38 @@ impl From<&str> for Direction {
     }
 }
 
-impl From<Direction> for &str {
-    fn from(d: Direction) -> Self {
-        match d {
+impl Direction {
+    /// Convert direction to its string representation
+    pub const fn as_str(&self) -> &'static str {
+        match self {
             Direction::Before => "~",
             Direction::After => "+",
         }
     }
 }
 
+impl From<Direction> for &str {
+    fn from(d: Direction) -> Self {
+        d.as_str()
+    }
+}
+
 /// Get the revision from a version string.
+///
+/// # Example
+/// ```
+/// use debversion::upstream::{get_revision, Direction, VcsSnapshot};
+/// assert_eq!(get_revision("1.2.3+bzr123"), ("1.2.3", Some((Direction::After, VcsSnapshot::Bzr
+/// { revno: "123".to_string() }))));
+/// assert_eq!(get_revision("1.2.3+git20210101.abcdefa"), ("1.2.3",
+/// Some((Direction::After, VcsSnapshot::Git { date: Some(chrono::NaiveDate::from_ymd_opt(2021, 1,
+/// 1).unwrap()), sha: Some("abcdefa".to_string()), snapshot: None }))));
+/// assert_eq!(get_revision("1.2.3+git20210101.1.abcdefa"), ("1.2.3",
+/// Some((Direction::After, VcsSnapshot::Git { date: Some(chrono::NaiveDate::from_ymd_opt(2021, 1,
+/// 1).unwrap()), sha: Some("abcdefa".to_string()), snapshot: Some(1) }))));
+/// assert_eq!(get_revision("1.2.3+svn123"), ("1.2.3", Some((Direction::After,
+/// VcsSnapshot::Svn { revno: 123 }))));
+/// ```
 pub fn get_revision(version_string: &str) -> (&str, Option<(Direction, VcsSnapshot)>) {
     if let Some((_, b, s, r)) =
         lazy_regex::regex_captures!(r"^(.*)([\+~])bzr(\d+)$", version_string)
@@ -187,6 +227,21 @@ pub fn get_revision(version_string: &str) -> (&str, Option<(Direction, VcsSnapsh
 /// * `version_string` - Original version string
 /// * `sep` - Separator to use when adding snapshot
 /// * `vcs_snapshot` - VCS snapshot information
+///
+/// # Example
+/// ```
+/// use debversion::upstream::{upstream_version_add_revision, VcsSnapshot};
+/// assert_eq!(upstream_version_add_revision("1.2.3", VcsSnapshot::Bzr { revno:
+/// "123".to_string() }, None), "1.2.3+bzr123");
+/// assert_eq!(upstream_version_add_revision("1.2.3+bzr123", VcsSnapshot::Bzr { revno:
+/// "124".to_string() }, None), "1.2.3+bzr124");
+/// assert_eq!(upstream_version_add_revision("1.2.3", VcsSnapshot::Git { date:
+/// Some(chrono::NaiveDate::from_ymd_opt(2021, 1, 1).unwrap()), sha: None, snapshot: None }, None),
+/// "1.2.3+git20210101");
+/// assert_eq!(upstream_version_add_revision("1.2.3+git20210101.abcdefa",
+/// VcsSnapshot::Git { date: None, sha: Some("abcdefa".to_string()), snapshot: None }, None),
+/// "1.2.3+gitabcdefa");
+/// ```
 pub fn upstream_version_add_revision(
     version_string: &str,
     mut vcs_snapshot: VcsSnapshot,
@@ -229,7 +284,7 @@ pub fn upstream_version_add_revision(
         }
     }
 
-    let sep: &str = sep.into();
+    let sep = sep.as_str();
 
     format!("{}{}{}", base_version, sep, vcs_snapshot.to_suffix())
 }
@@ -278,28 +333,28 @@ mod tests {
             .to_suffix()
         );
         assert_eq!(
-            "gitabcdefg",
+            "gitabcdefa",
             super::VcsSnapshot::Git {
                 date: None,
-                sha: Some("abcdefg".to_string()),
+                sha: Some("abcdefa".to_string()),
                 snapshot: None,
             }
             .to_suffix()
         );
         assert_eq!(
-            "git20210101.abcdefg",
+            "git20210101.abcdefa",
             super::VcsSnapshot::Git {
                 date: Some(chrono::NaiveDate::from_ymd_opt(2021, 1, 1).unwrap()),
-                sha: Some("abcdefg".to_string()),
+                sha: Some("abcdefa".to_string()),
                 snapshot: None,
             }
             .to_suffix()
         );
         assert_eq!(
-            "git20210101.1.abcdefg",
+            "git20210101.1.abcdefa",
             super::VcsSnapshot::Git {
                 date: Some(chrono::NaiveDate::from_ymd_opt(2021, 1, 1).unwrap()),
-                sha: Some("abcdefg".to_string()),
+                sha: Some("abcdefa".to_string()),
                 snapshot: Some(1),
             }
             .to_suffix()
@@ -412,4 +467,59 @@ mod tests {
             )
         );
     }
+
+    #[test]
+    fn test_upstream_version_add_revision_git_snapshot_increment() {
+        // Test the missed mutants in line 273-274 around snapshot increment logic
+
+        // Test when dates are equal but SHAs are different - should increment snapshot
+        let result = super::upstream_version_add_revision(
+            "1.2.3+git20210101.1.abcdefa",
+            super::VcsSnapshot::Git {
+                date: Some(chrono::NaiveDate::from_ymd_opt(2021, 1, 1).unwrap()),
+                sha: Some("bcdefgh".to_string()),
+                snapshot: None,
+            },
+            None,
+        );
+        // The SHA is truncated to 7 characters, so "bcdefgh" becomes "bcdefgh"
+        assert_eq!("1.2.3+git20210101.2.bcdefgh", result);
+
+        // Test when dates are different - should reset snapshot to 1
+        let result2 = super::upstream_version_add_revision(
+            "1.2.3+git20210101.5.abcdefa",
+            super::VcsSnapshot::Git {
+                date: Some(chrono::NaiveDate::from_ymd_opt(2021, 1, 2).unwrap()),
+                sha: Some("bcdefgh".to_string()),
+                snapshot: None,
+            },
+            None,
+        );
+        assert_eq!("1.2.3+git20210102.1.bcdefgh", result2);
+
+        // Test when dates are equal and SHAs are equal - should reset snapshot to 1
+        let result3 = super::upstream_version_add_revision(
+            "1.2.3+git20210101.1.abcdefa",
+            super::VcsSnapshot::Git {
+                date: Some(chrono::NaiveDate::from_ymd_opt(2021, 1, 1).unwrap()),
+                sha: Some("abcdefa".to_string()),
+                snapshot: None,
+            },
+            None,
+        );
+        // Since date == date and sha == sha (NOT !=), condition is false, so snapshot = Some(1)
+        assert_eq!("1.2.3+git20210101.1.abcdefa", result3);
+
+        // Test different dates should set snapshot to 1 (not increment)
+        let result4 = super::upstream_version_add_revision(
+            "1.2.3+git20210101.3.abcdefa",
+            super::VcsSnapshot::Git {
+                date: Some(chrono::NaiveDate::from_ymd_opt(2021, 1, 3).unwrap()),
+                sha: Some("bcdefgh".to_string()),
+                snapshot: None,
+            },
+            None,
+        );
+        assert_eq!("1.2.3+git20210103.1.bcdefgh", result4);
+    }
 }
